package com.pms.ocp.biz.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.pms.ocp.biz.BizModelDemandService;
import com.pms.ocp.model.entity.*;
import com.pms.ocp.model.dto.ModelDemandDTO;
import com.pms.ocp.service.ModelBaseVersionService;
import com.pms.ocp.service.ModelDemandService;
import com.pms.ocp.service.ModelPropertyVersionService;
import com.pms.ocp.service.OcpModelGroupService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * @Auther: wangjian
 * @Date: 2022/4/1 11:19
 * @Description:
 */

@Service
public class BizModelDemandServiceImpl implements BizModelDemandService {

    @Autowired
    private ModelDemandService modelDemandService;

    @Autowired
    private ModelBaseVersionService modelBaseVersionService;

    @Autowired
    private ModelPropertyVersionService modelPropertyVersionService;


    @Transactional
    @Override
    public Integer createModelDemand(ModelDemandDTO modelDemandDTO) {

        ModelDemand modelDemand = new ModelDemand();
        BeanUtils.copyProperties(modelDemandDTO, modelDemand);
        modelDemand.setApplyCtime(LocalDateTime.now());

        if (modelDemandDTO.getTargetType().equals("1")) {
            ModelBaseVersion modelBaseVersion = new ModelBaseVersion();
            BeanUtils.copyProperties(modelDemandDTO, modelBaseVersion);
            modelBaseVersionService.createModelBaseVersion(modelBaseVersion);
        } else {
            ModelPropertyVersion modelPropertyVersion = new ModelPropertyVersion();
            BeanUtils.copyProperties(modelDemandDTO, modelPropertyVersion);
            modelPropertyVersionService.createModelPropertyVersion(modelPropertyVersion);
        }

        modelDemandService.createModelDemand(modelDemand);

        return 1;
    }

    @Transactional
    @Override
    public Integer updateModelDemand(ModelDemandDTO modelDemandDTO) {
        ModelDemand modelDemand = new ModelDemand();
        BeanUtils.copyProperties(modelDemandDTO, modelDemand);
        modelDemand.setApplyCtime(LocalDateTime.now());

        if (modelDemandDTO.getTargetType().equals("1")) {
            ModelBaseVersion modelBaseVersion = new ModelBaseVersion();
            BeanUtils.copyProperties(modelDemandDTO, modelBaseVersion);
            modelBaseVersionService.updateModelBaseVersion(modelBaseVersion);
        } else {
            ModelPropertyVersion modelPropertyVersion = new ModelPropertyVersion();
            BeanUtils.copyProperties(modelDemandDTO, modelPropertyVersion);
            modelPropertyVersionService.updateModelPropertyVersion(modelPropertyVersion);
        }

        modelDemandService.updateModelDemand(modelDemand);
        return 1;
    }

    @Transactional
    @Override
    public Integer deleteModelDemand(String targetType, String demandId, String modelId, String proId) {

        modelDemandService.deleteModelDemand(modelId, demandId);

        if (StringUtils.isNotEmpty(modelId)) {

            modelBaseVersionService.deleteModelBaseVersion(modelId, demandId);
        }

        if (StringUtils.isNotEmpty(proId)) {

            modelPropertyVersionService.deleteModelPropertyVersion(demandId, proId);
        }

        return 1;
    }


    @Autowired
    private OcpModelGroupService ocpModelGroupService;

    @Override
    public Model getModelDemand(String objId, String modelGroupCode) {
        return null;
    }

//    public List<String> getGroupCodeRecursion(String modelGroupCode) {
//
//
//        List<String> modelGroupCodeList = new ArrayList<>();
//        if (StringUtils.isNotEmpty(modelGroupCode)) {
//
//            modelGroupCodeList.add(modelGroupCode);
//
//            QueryWrapper<ModelClassify> queryWrapper = new QueryWrapper();
//            if (StringUtils.isNotEmpty(modelGroupCode)) {
//                queryWrapper.lambda().eq(ModelClassify::getModelGroupPcode, modelGroupCode);
//            }
//            List<ModelClassify> list = ocpModelGroupService.list(queryWrapper);
//
//            if (!list.isEmpty()) {
//                for (ModelClassify modelClassify : list) {
//
//                    String modelGroupCode1 = modelClassify.getModelGroupCode();
//                    modelGroupCodeList.add(modelGroupCode1);
//
//                    QueryWrapper<ModelClassify> queryWrapper1 = new QueryWrapper();
//
//                    if (StringUtils.isNotEmpty(modelGroupCode1)) {
//                        queryWrapper1.lambda().eq(ModelClassify::getModelGroupPcode, modelGroupCode1);
//                    }
//                    List<ModelClassify> list2 = ocpModelGroupService.list(queryWrapper1);
//
//                    if (!list2.isEmpty()) {
//                        getGroupCodeRecursion(modelGroupCode1);
//                    }
//
//                }
//            }
//
//        }
//
//        return modelGroupCodeList;
//    }


    @Override
    public List<Model> getModelDemandList(String objId, String modelGroupCode, int currentPage, int pageSize) {

        List<String> modelGroupCodeList = new ArrayList<>();

        QueryWrapper<ModelClassify> queryWrapper = new QueryWrapper();
        List<ModelClassify> modelClassifyList = ocpModelGroupService.list(queryWrapper);

        getParentNode(modelGroupCodeList, modelClassifyList, modelGroupCode);
        modelGroupCodeList.add(modelGroupCode);
        modelGroupCodeList.size();



//        List<ModelBaseVersion> modelBaseVersionList = modelBaseVersionService.getModelBaseVersionList(modelGroupCodeList, currentPage, pageSize);

        return null;
    }

    /**
     * 递归获取某个父节点下面的所有子节点
     *
     * @param childMenu       要返回的结果
     * @param menuList        数据库查询出来的所有集合
     * @param modelGroupPcode 父类编码
     */
    private static void getParentNode(List<String> childMenu, List<ModelClassify> menuList, String modelGroupPcode) {
        menuList.stream()
                //过滤出父id等于参数的id
                .filter(menu -> StringUtils.isNotBlank(menu.getModelGroupPcode()) && menu.getModelGroupPcode().equals(modelGroupPcode))
                .forEach(menu -> {
                    //递归遍历下一级
                    getParentNode(childMenu, menuList, menu.getModelGroupCode());
                    //添加
                    childMenu.add(menu.getModelGroupCode());
                });
    }

}
